////
/// Copyright (c) 2016-2022 Martin Donath <martin.donath@squidfunk.com>
///
/// Permission is hereby granted, free of charge, to any person obtaining a
/// copy of this software and associated documentation files (the "Software"),
/// to deal in the Software without restriction, including without limitation
/// the rights to use, copy, modify, merge, publish, distribute, sublicense,
/// and/or sell copies of the Software, and to permit persons to whom the
/// Software is furnished to do so, subject to the following conditions:
///
/// The above copyright notice and this permission notice shall be included in
/// all copies or substantial portions of the Software.
///
/// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
/// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
/// FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
/// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
/// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
/// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
/// DEALINGS
////

// ----------------------------------------------------------------------------
// Rules
// ----------------------------------------------------------------------------

// Footnotes variables
:root {
  --md-footnotes-icon: svg-load("material/keyboard-return.svg");
}

// ----------------------------------------------------------------------------

// Scoped in typesetted content to match specificity of regular content
.md-typeset {

  // Footnote container
  .footnote {
    color: var(--md-default-fg-color--light);
    font-size: px2rem(12.8px);

    // Footnote list - omit left indentation
    > ol {
      margin-inline-start: 0;

      // Footnote item - footnote items can contain lists, so we need to scope
      // the spacing adjustments to the top-level footnote item.
      > li {
        transition: color 125ms;

        // Darken color on target
        &:target {
          color: var(--md-default-fg-color);
        }

        // Show backreferences on footnote focus without transition
        &:focus-within .footnote-backref {
          transform: translateX(0);
          opacity: 1;
          transition: none;
        }

        // Show backreferences on footnote hover/target
        &:is(:hover, :target) .footnote-backref {
          transform: translateX(0);
          opacity: 1;
        }

        // Adjust spacing on first child
        > :first-child {
          margin-top: 0;
        }
      }
    }
  }

  // Footnote reference
  .footnote-ref {
    font-weight: 700;
    font-size: px2em(12px, 16px);

    // Hack: increase specificity to override default
    html & {
      outline-offset: px2rem(2px);
    }
  }

  // Show outline for all devices
  [id^="fnref:"]:target > .footnote-ref {
    outline: auto;
  }

  // Footnote backreference
  .footnote-backref {
    display: inline-block;
    color: var(--md-typeset-a-color);
    // Hack: omit Unicode arrow for replacement with icon
    font-size: 0;
    vertical-align: text-bottom;
    transform: translateX(px2rem(5px));
    opacity: 0;
    transition:
      color     250ms,
      transform 250ms 250ms,
      opacity   125ms 250ms;

    // [print]: Show footnote backreferences
    @media print {
      color: var(--md-typeset-a-color);
      transform: translateX(0);
      opacity: 1;
    }

    // Adjust for right-to-left languages
    [dir="rtl"] & {
      transform: translateX(px2rem(-5px));
    }

    // Adjust color on hover
    &:hover {
      color: var(--md-accent-fg-color);
    }

    // Footnote backreference icon
    &::before {
      display: inline-block;
      width: px2rem(16px);
      height: px2rem(16px);
      background-color: currentcolor;
      mask-image: var(--md-footnotes-icon);
      mask-position: center;
      mask-repeat: no-repeat;
      mask-size: contain;
      content: "";

      // Adjust for right-to-left languages
      [dir="rtl"] & {

        // Flip icon vertically
        svg {
          transform: scaleX(-1);
        }
      }
    }
  }
}
