////
/// Copyright (c) 2016-2022 Martin Donath <martin.donath@squidfunk.com>
///
/// Permission is hereby granted, free of charge, to any person obtaining a
/// copy of this software and associated documentation files (the "Software"),
/// to deal in the Software without restriction, including without limitation
/// the rights to use, copy, modify, merge, publish, distribute, sublicense,
/// and/or sell copies of the Software, and to permit persons to whom the
/// Software is furnished to do so, subject to the following conditions:
///
/// The above copyright notice and this permission notice shall be included in
/// all copies or substantial portions of the Software.
///
/// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
/// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
/// FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
/// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
/// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
/// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
/// DEALINGS
////

@use "sass:math";

// ----------------------------------------------------------------------------
// Helpers
// ----------------------------------------------------------------------------

///
/// Strip units from a number
///
@function strip-units($number) {
  @return math.div($number, ($number * 0 + 1));
}

///
/// Convert color in HEX to HSL
///
/// Note, that we need to strip the `deg` units from the `hue` value, as they
/// were added in Color Level 4, which not all browsers support.
///
@function hex2hsl($color) {
  @return
    round(strip-units(hue($color))),
    round(saturation($color)),
    round(lightness($color));
}

// ----------------------------------------------------------------------------

///
/// Convert font size in px to em
///
@function px2em($size, $base: 16px) {
  @if unit($size) == px {
    @if unit($base) == px {
      @return math.div($size, $base) * 1em;
    } @else {
      @error "Invalid base: #{$base} - unit must be 'px'";
    }
  } @else {
    @error "Invalid size: #{$size} - unit must be 'px'";
  }
}

///
/// Convert font size in px to rem
///
@function px2rem($size, $base: 20px) {
  @if unit($size) == px {
    @if unit($base) == px {
      @return math.div($size, $base) * 1rem;
    } @else {
      @error "Invalid base: #{$base} - unit must be 'px'";
    }
  } @else {
    @error "Invalid size: #{$size} - unit must be 'px'";
  }
}
