{#-
  This file was automatically generated - do not edit
-#}
{% macro render(nav_item, path, level) %}
  {% set class = "md-nav__item" %}
  {% if nav_item.active %}
    {% set class = class ~ " md-nav__item--active" %}
  {% endif %}
  {%- set orig_class = class %}
  {%- if nav_item.current %}
    {% set class = class ~ " md-nav__current-nested" %}
  {%- endif %}
  {% if nav_item.children %}
    {% if "navigation.sections" in features and level == 1 + (
      "navigation.tabs" in features
    ) %}
      {% set class = class ~ " md-nav__item--section" %}
    {% endif %}
    <li class="{{ class }} md-nav__item--nested">
      {% set checked = "checked" if nav_item.active %}
      {% if "navigation.expand" in features and not checked %}
        <input class="md-nav__toggle md-toggle md-toggle--indeterminate" data-md-toggle="{{ path }}" type="checkbox" id="{{ path }}" checked>
      {% else %}
        <input class="md-nav__toggle md-toggle" data-md-toggle="{{ path }}" type="checkbox" id="{{ path }}" {{ checked }}>
      {% endif %}
      {% set indexes = [] %}
      {% if nav_item.caption_only %}
        <label class="md-nav__link" for="{{ path }}">
          {{ nav_item.title }}
          <span class="md-nav__icon md-icon"></span>
        </label>
      {% else %}
        {% set class = "md-nav__link--active" if nav_item.current %}
        <div class="md-nav__link md-nav__link--index {{ class }}">
          <a href="{{ nav_item.url|e }}">{{ nav_item.title }}</a>
          {% if True %}
            <label for="{{ path }}">
              <span class="md-nav__icon md-icon"></span>
            </label>
          {% endif %}
        </div>
      {% endif %}
      <nav class="md-nav" aria-label="{{ nav_item.aria_label }}" data-md-level="{{ level }}">
        <label class="md-nav__title" for="{{ path }}">
          <span class="md-nav__icon md-icon"></span>
          {{ nav_item.title }}
        </label>
        <ul class="md-nav__list" data-md-scrollfix>
          {% for nav_item in nav_item.children %}
            {% if not indexes or nav_item != indexes | first %}
              {{ render(nav_item, path ~ "_" ~ loop.index, level + 1) }}
            {% endif %}
          {% endfor %}
        </ul>
      </nav>
    </li>
  {% endif %}
  {% if nav_item.current %}
    {%- set class = orig_class %}
    {%- if nav_item.children %}
      {% set class = class ~ " md-nav__current-toc" %}
    {%- endif %}
    <li class="{{ class }}">
      {% set toc = page.integrated_local_toc %}
      {% set use_integrated_local_toc = True %}
      <input class="md-nav__toggle md-toggle" data-md-toggle="toc" type="checkbox" id="__toc">
      {% if toc | first is defined %}
        <label class="md-nav__link md-nav__link--active" for="__toc">
          {{ nav_item.title }}
          <span class="md-nav__icon md-icon"></span>
        </label>
      {% endif %}
      <a href="{{ nav_item.url | url }}" class="md-nav__link md-nav__link--active">
        {{ nav_item.title }}
      </a>
      {% if toc | first is defined %}
        {% include "partials/toc.html" %}
      {% endif %}
    </li>
  {% elif not nav_item.children %}
    <li class="{{ class }}">
      <a href="{{ nav_item.url | url }}" class="md-nav__link">
        {{ nav_item.title }}
      </a>
    </li>
  {% endif %}
{% endmacro %}
{{ render(nav_item, path, level) }}
