/* custom CSS for tweaked examples about code annotations */
.very-large-tooltip {
  --md-tooltip-width: 600px;
}

.annotated-with-numbers .md-annotation__index > ::before {
  content: attr(data-md-annotation-id);
}

.annotated-with-numbers :focus-within > .md-annotation__index > ::before {
  transform: none;
}

/* ************************* animated-admonition-border style ******************************** */
@keyframes flash_border {
  0% {
    border-color: blueviolet;
  }
  50% {
    border-color: aqua;
  }
  100% {
    border-color: blueviolet;
  }
}

.md-typeset .admonition.animated-admonition-border {
  animation: flash_border 1.5s infinite;
}

/* ************************* my-special-key style ************************************* */

.md-typeset kbd.key-my-special-key,
.md-typeset kbd.key-git {
  color: var(--md-primary-fg-color);
  background: var(--md-accent-bg-color);
}

.md-typeset kbd.key-git::before {
  content: "██";
  -webkit-mask-image: var(--si-icon--fontawesome_brands_git);
  -webkit-mask-repeat: no-repeat;
  mask-image: var(--si-icon--fontawesome_brands_git);
  mask-repeat: no-repeat;
}

.md-typeset kbd.key-my-special-key:hover,
.md-typeset kbd.key-git:hover {
  color: var(--md-accent-fg-color);
}

/* **************************** custom-task-list style rules *************************** */

/* style for the unchecked checkboxes */
.custom-task-list-style > ul > li > label .task-list-indicator::before {
  -webkit-mask-image: var(--md-admonition-icon--failure);
  mask-image: var(--md-admonition-icon--failure);
  background-color: hsl(0, 72%, 55%);
}

/* style for the checked checkboxes */
.custom-task-list-style > ul > li > .task-list-control > [type="checkbox"]:checked + .task-list-indicator::before {
  -webkit-mask-image: var(--md-admonition-icon--success);
  mask-image: var(--md-admonition-icon--success);
  background-color: hsl(122, 84%, 45%);
}

/* *************************** custom admonition style rules *************************** */

:root {
  --md-admonition-icon--pied-piper: url('data:image/svg+xml;charset=utf-8,<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 576 512"><path d="M244 246c-3.2-2-6.3-2.9-10.1-2.9-6.6 0-12.6 3.2-19.3 3.7l1.7 4.9zm135.9 197.9c-19 0-64.1 9.5-79.9 19.8l6.9 45.1c35.7 6.1 70.1 3.6 106-9.8-4.8-10-23.5-55.1-33-55.1zM340.8 177c6.6 2.8 11.5 9.2 22.7 22.1 2-1.4 7.5-5.2 7.5-8.6 0-4.9-11.8-13.2-13.2-23 11.2-5.7 25.2-6 37.6-8.9 68.1-16.4 116.3-52.9 146.8-116.7C548.3 29.3 554 16.1 554.6 2l-2 2.6c-28.4 50-33 63.2-81.3 100-31.9 24.4-69.2 40.2-106.6 54.6l-6.3-.3v-21.8c-19.6 1.6-19.7-14.6-31.6-23-18.7 20.6-31.6 40.8-58.9 51.1-12.7 4.8-19.6 10-25.9 21.8 34.9-16.4 91.2-13.5 98.8-10zM555.5 0l-.6 1.1-.3.9.6-.6zm-59.2 382.1c-33.9-56.9-75.3-118.4-150-115.5l-.3-6c-1.1-13.5 32.8 3.2 35.1-31l-14.4 7.2c-19.8-45.7-8.6-54.3-65.5-54.3-14.7 0-26.7 1.7-41.4 4.6 2.9 18.6 2.2 36.7-10.9 50.3l19.5 5.5c-1.7 3.2-2.9 6.3-2.9 9.8 0 21 42.8 2.9 42.8 33.6 0 18.4-36.8 60.1-54.9 60.1-8 0-53.7-50-53.4-60.1l.3-4.6 52.3-11.5c13-2.6 12.3-22.7-2.9-22.7-3.7 0-43.1 9.2-49.4 10.6-2-5.2-7.5-14.1-13.8-14.1-3.2 0-6.3 3.2-9.5 4-9.2 2.6-31 2.9-21.5 20.1L15.9 298.5c-5.5 1.1-8.9 6.3-8.9 11.8 0 6 5.5 10.9 11.5 10.9 8 0 131.3-28.4 147.4-32.2 2.6 3.2 4.6 6.3 7.8 8.6 20.1 14.4 59.8 85.9 76.4 85.9 24.1 0 58-22.4 71.3-41.9 3.2-4.3 6.9-7.5 12.4-6.9.6 13.8-31.6 34.2-33 43.7-1.4 10.2-1 35.2-.3 41.1 26.7 8.1 52-3.6 77.9-2.9 4.3-21 10.6-41.9 9.8-63.5l-.3-9.5c-1.4-34.2-10.9-38.5-34.8-58.6-1.1-1.1-2.6-2.6-3.7-4 2.2-1.4 1.1-1 4.6-1.7 88.5 0 56.3 183.6 111.5 229.9 33.1-15 72.5-27.9 103.5-47.2-29-25.6-52.6-45.7-72.7-79.9zm-196.2 46.1v27.2l11.8-3.4-2.9-23.8zm-68.7-150.4l24.1 61.2 21-13.8-31.3-50.9zm84.4 154.9l2 12.4c9-1.5 58.4-6.6 58.4-14.1 0-1.4-.6-3.2-.9-4.6-26.8 0-36.9 3.8-59.5 6.3z"/></svg>')
}

.md-typeset .admonition.pied-piper {
  border-color: rgb(43, 155, 70);
}

.md-typeset .pied-piper>.admonition-title {
  background-color: rgba(43, 155, 70, 0.1);
  border-color: rgb(43, 155, 70);
}

.md-typeset .pied-piper>.admonition-title::before {
  background-color: rgb(43, 155, 70);
  -webkit-mask-image: var(--md-admonition-icon--pied-piper);
  mask-image: var(--md-admonition-icon--pied-piper);
}

/* ************************ custom-tab-set-style **************************** */

.custom-tab-set-style {
  border: solid 2px var(--md-default-fg-color);
  padding: 0 5px;
}

/* *********************** custom-tab-item-style **************************** */

.custom-tab-item-style {
  border: solid 2px var(--md-default-fg-color);
  padding: 0 5px;
  margin-top: 3px;
}

/* *********************** inline icon pulsing-heart style **************************** */
@keyframes heart {
  0%, 40%, 80%, 100% {
    transform: scale(1);
  }
  20%, 60% {
    transform: scale(1.15);
  }
}

.pulsing-heart::before {
  background-color: red;
  animation: heart 1s infinite;
  height: 1.5em;
  width: 1.5em;
  vertical-align: middle;
}

/* ************************* badge style inline stuff ****************************** */

.inline-badge,
.red,
.pink,
.purple,
.deep-purple,
.indigo,
.blue,
.light-blue,
.cyan,
.teal,
.green,
.light-green,
.lime,
.yellow,
.amber,
.orange,
.deep-orange,
.brown,
.grey,
.blue-grey,
.white,
.black,
.accent-red,
.accent-pink,
.accent-purple,
.accent-deep-purple,
.accent-indigo,
.accent-blue,
.accent-light-blue,
.accent-cyan,
.accent-teal,
.accent-green,
.accent-light-green,
.accent-lime,
.accent-yellow,
.accent-amber,
.accent-orange,
.accent-deep-orange {
  padding: 0.05rem 0.3rem;
  border-radius: 0.25rem;
}

.red {
  background-color: #ef5552;
  color: var(--md-code-fg-color);
}

.pink {
  background-color: #e92063;
  color: var(--md-code-fg-color);
}

.purple {
  background-color: #ab47bd;
  color: var(--md-code-fg-color);
}

.deep-purple {
  background-color: #7e56c2;
  color: var(--md-code-fg-color);
}

.indigo {
  background-color: #4051b5;
  color: var(--md-code-fg-color);
}

.blue {
  background-color: #2094f3;
  color: var(--md-code-fg-color);
}

.light-blue {
  background-color: #02a6f2;
  color: var(--md-code-fg-color);
}

.cyan {
  background-color: #00bdd6;
  color: var(--md-code-fg-color);
}

.teal {
  background-color: #009485;
  color: var(--md-code-fg-color);
}

.green {
  background-color: #4cae4f;
  color: var(--md-code-fg-color);
}

.light-green {
  background-color: #8bc34b;
  color: var(--md-code-fg-color);
}

.lime {
  background-color: #cbdc38;
  color: var(--md-code-bg-color);
}

.yellow {
  background-color: #ffec3d;
  color: var(--md-code-bg-color);
}

.amber {
  background-color: #ffc105;
  color: var(--md-code-bg-color);
}

.orange {
  background-color: #ffa724;
  color: var(--md-code-bg-color);
}

.deep-orange {
  background-color: #ff6e42;
  color: var(--md-code-fg-color);
}

.brown {
  background-color: #795649;
  color: var(--md-code-fg-color);
}

.grey {
  background-color: #757575;
  color: var(--md-code-fg-color);
}

.blue-grey {
  background-color: #546d78;
  color: var(--md-code-fg-color);
}

.white {
  background-color: #fff;
  color: #000;
}

.black {
  background-color: #000;
  color: #fff;
}

.accent-red {
  color: #ef5552;
  background-color: var(--md-code-bg-color)
}

.accent-pink {
  color: #e92063;
  background-color: var(--md-code-bg-color)
}

.accent-purple {
  color: #ab47bd;
  background-color: var(--md-code-bg-color)
}

.accent-deep-purple {
  color: #7e56c2;
  background-color: var(--md-code-bg-color)
}

.accent-indigo {
  color: #4051b5;
  background-color: var(--md-code-bg-color)
}

.accent-blue {
  color: #2094f3;
  background-color: var(--md-code-bg-color)
}

.accent-light-blue {
  color: #02a6f2;
  background-color: var(--md-code-bg-color)
}

.accent-cyan {
  color: #00bdd6;
  background-color: var(--md-code-bg-color)
}

.accent-teal {
  color: #009485;
  background-color: var(--md-code-bg-color)
}

.accent-green {
  color: #4cae4f;
  background-color: var(--md-code-bg-color)
}

.accent-light-green {
  color: #8bc34b;
  background-color: var(--md-code-bg-color)
}

.accent-lime {
  color: #cbdc38;
  background-color: var(--md-code-bg-color)
}

.accent-yellow {
  color: #ffec3d;
  background-color: var(--md-code-bg-color)
}

.accent-amber {
  color: #ffc105;
  background-color: var(--md-code-bg-color)
}

.accent-orange {
  color: #ffa724;
  background-color: var(--md-code-bg-color)
}

.accent-deep-orange {
  color: #ff6e42;
  background-color: var(--md-code-bg-color)
}