{#-
  This file was automatically generated - do not edit
-#}
{% if page.edit_url %}
  <a href="{{ page.edit_url }}" title="{{ lang.t('edit.link.title') }}" class="md-content__button md-icon">
    {% set icon = config.theme.icon.edit or "material/pencil" %}
    {% include ".icons/" ~ icon ~ ".svg" %}
  </a>
{% endif %}
