<!--
  Copyright (c) 2016-2022 Martin Donath <martin.donath@squidfunk.com>

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to
  deal in the Software without restriction, including without limitation the
  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
  sell copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
  IN THE SOFTWARE.
-->

<!-- Translations: Chinese (Simplified) -->
{% macro t(key) %}{{ {
  "language": "zh",
  "announce.dismiss": "不再显示此消息",
  "clipboard.copy": "复制",
  "clipboard.copied": "已复制",
  "consent.accept": "同意",
  "consent.manage": "管理设定",
  "consent.reject": "拒绝",
  "edit.link.title": "编辑此页",
  "footer.previous": "上一页",
  "footer.next": "下一页",
  "footer.title": "页脚",
  "header.title": "页眉",
  "meta.comments": "评论",
  "meta.source": "来源",
  "nav.title": "导航栏",
  "search.config.lang": "ja",
  "search.config.pipeline": "trimmer, stemmer",
  "search.config.separator": "[\\s\\-，。]+",
  "search.placeholder": "搜索",
  "search.share": "分享",
  "search.reset": "清空当前内容",
  "search.result.initializer": "正在初始化搜索引擎",
  "search.result.placeholder": "键入以开始搜索",
  "search.result.none": "没有找到符合条件的结果",
  "search.result.one": "找到 1 个符合条件的结果",
  "search.result.other": "# 个符合条件的结果",
  "search.result.more.one": "在该页上还有 1 个符合条件的结果",
  "search.result.more.other": "在该页上还有 # 个符合条件的结果",
  "search.result.term.missing": "缺少",
  "search.title": "查找",
  "select.language.title": "选择当前语言",
  "select.version.title": "选择当前版本",
  "skip.link.title": "跳转至",
  "source.link.title": "前往仓库",
  "source.file.date.updated": "最后更新",
  "source.file.date.created": "创建日期",
  "tabs.title": "标签",
  "toc.title": "目录",
  "top.title": "回到页面顶部"
}[key] }}{% endmacro %}
