////
/// Copyright (c) 2016-2022 Martin Donath <martin.donath@squidfunk.com>
///
/// Permission is hereby granted, free of charge, to any person obtaining a
/// copy of this software and associated documentation files (the "Software"),
/// to deal in the Software without restriction, including without limitation
/// the rights to use, copy, modify, merge, publish, distribute, sublicense,
/// and/or sell copies of the Software, and to permit persons to whom the
/// Software is furnished to do so, subject to the following conditions:
///
/// The above copyright notice and this permission notice shall be included in
/// all copies or substantial portions of the Software.
///
/// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
/// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
/// FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
/// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
/// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
/// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
/// DEALINGS
////

// ----------------------------------------------------------------------------
// Rules
// ----------------------------------------------------------------------------

// Tasklist variables
:root {
  --md-tasklist-icon:
    svg-load("octicons/check-circle-fill-24.svg");
  --md-tasklist-icon--checked:
    svg-load("octicons/check-circle-fill-24.svg");
}

// ----------------------------------------------------------------------------

// Scoped in typesetted content to match specificity of regular content
.md-typeset {

  // Tasklist item
  .task-list-item {
    position: relative;
    list-style-type: none;

    // Make checkbox items align with normal list items, but position
    // everything in ems for correct layout at smaller font sizes
    [type="checkbox"] {
      position: absolute;
      top: 0.45em;
      inset-inline-start: -2em;
    }
  }

  // Hide native checkbox, when custom classes are enabled
  .task-list-control [type="checkbox"] {
    z-index: -1;
    opacity: 0;
  }

  // Tasklist indicator in unchecked state
  .task-list-indicator::before {
    position: absolute;
    top: 0.15em;
    inset-inline-start: px2em(-24px);
    width: px2em(20px);
    height: px2em(20px);
    background-color: var(--md-default-fg-color--lightest);
    mask-image: var(--md-tasklist-icon);
    mask-position: center;
    mask-repeat: no-repeat;
    mask-size: contain;
    content: "";
  }

  // Tasklist indicator in checked state
  [type="checkbox"]:checked + .task-list-indicator::before {
    background-color: $clr-green-a400;
    mask-image: var(--md-tasklist-icon--checked);
  }
}
