////
/// Copyright (c) 2016-2022 Martin Donath <martin.donath@squidfunk.com>
///
/// Permission is hereby granted, free of charge, to any person obtaining a
/// copy of this software and associated documentation files (the "Software"),
/// to deal in the Software without restriction, including without limitation
/// the rights to use, copy, modify, merge, publish, distribute, sublicense,
/// and/or sell copies of the Software, and to permit persons to whom the
/// Software is furnished to do so, subject to the following conditions:
///
/// The above copyright notice and this permission notice shall be included in
/// all copies or substantial portions of the Software.
///
/// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
/// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
/// FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
/// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
/// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
/// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
/// DEALINGS
////

// ----------------------------------------------------------------------------
// Rules
// ----------------------------------------------------------------------------

// Scoped in typesetted content to match specificity of regular content
.md-typeset {

  // Deletion, addition or comment
  :is(del, ins, .comment).critic {
    box-decoration-break: clone;
  }

  // Deletion
  del.critic {
    background-color: var(--md-typeset-del-color);
  }

  // Addition
  ins.critic {
    background-color: var(--md-typeset-ins-color);
  }

  // Comment
  .critic.comment {
    color: var(--md-code-hl-comment-color);

    // Comment opening mark
    &::before {
      content: "/* ";
    }

    // Comment closing mark
    &::after {
      content: " */";
    }
  }

  // Critic block
  .critic.block {
    display: block;
    margin: 1em 0;
    padding-inline: px2rem(16px);
    overflow: auto;
    box-shadow: none;

    // Adjust spacing on first child
    > :first-child {
      margin-top: 0.5em;
    }

    // Adjust spacing on last child
    > :last-child {
      margin-bottom: 0.5em;
    }
  }
}
